@@ -12,7 +12,7 @@ Example:
 >>> test1.delete()  # dangerous!
 '''
 
-__version__      = '0.2'
+__version__      = '0.3'
 __author__       = 'Volker Grabsch'
 __author_email__ = 'vog@notjusthosting.com'
 __url__          = 'http://www.profv.de/python-vserver/'
@@ -270,9 +270,10 @@ class VServer(object):
                     u'--hostname', self._name,
                     u'--interface', interface,
                     u'-m'] + method)
-        self.write(u'/etc/hosts', 0644,
-            u'%s  %s %s\n' % (ip, fqdn, self._name)
-        )
+        self.write_table(u'/etc/hosts', 0644, u'%s  %s', (
+            (u'127.0.0.1', u'localhost'),
+            (ip,           u'%s %s' % (fqdn, self._name))
+        ))
         self._write_cfg(u'/fstab', 0644,
             # disable ramdisk /tmp
             u'none  /proc     proc    defaults        0 0\n'
@@ -388,6 +389,19 @@ class DebianVServer(object):
         else:
             self.p.run([u'chpasswd'], input=(u'%s:%s\n' % (user, password)))
 
+    # apache
+    def set_apache_module_enabled(self, module, enabled):
+        if enabled:
+            self.p.run([u'a2enmod', module])
+        else:
+            self.p.run([u'a2dismod', module])
+
+    def set_apache_site_enabled(self, site, enabled):
+        if enabled:
+            self.p.run([u'a2ensite', site])
+        else:
+            self.p.run([u'a2dissite', site])
+
     # zope
     def add_zope_instance(self, version, user, zope_user, zope_password):
         self.p.run([u'su', u'-', user, u'-c',
@@ -426,13 +440,18 @@ class HostingVServer(object):
         self.p.timezone = timezone
         self.p.set_user_password(u'root', None)
         self.p.install(
-            u'less', u'vim', u'curl', u'w3m', u'rsync', u'mmv',
-            u'htop', u'mc', u'pwgen', u'bzip2', u'gcc', u'make',
+            u'less', u'vim', u'emacs', u'curl', u'w3m', u'rsync', u'mmv',
+            u'htop', u'mc', u'pwgen', u'bzip2', u'unzip',
+            u'gcc', u'g++', u'make',
             u'locales-all',
+            u'rcs',
+            u'cvs',
             u'subversion',
             u'mercurial', u'python-pygments',
             u'darcs',
             u'git-core', u'cogito',
+            u'netpbm', u'imagemagick',
+            u'ffmpeg', u'ffmpeg2theora',
         )
         running_services = self.p.running_services
         if len(running_services) != 4:
@@ -445,14 +464,139 @@ class HostingVServer(object):
     def install_ssh(self, keys_vserver=None):
         self.p.install(u'openssh-server', u'openssh-client')
         self.p.stop_service(u'ssh')
-        self.p.ssh_config = self.p.ssh_config + (
+        self.p.ssh_config = (
+            (u'Port',            u'22'),
+            (u'Protocol',        u'2'),
             (u'PermitRootLogin', u'no'),
+            (u'X11Forwarding',   u'yes'),
+            (u'PrintMotd',       u'no'),
             (u'UseDNS',          u'no'),
+            (u'ChallengeResponseAuthentication', u'no'),
+            (u'Subsystem', u'sftp /usr/lib/openssh/sftp-server'),
         )
         if keys_vserver is not None:
             self.p.ssh_host_keys = keys_vserver.p.ssh_host_keys
         self.p.start_service(u'ssh')
 
+    def install_tex(self):
+        self.p.install(
+            u'texlive',
+            u'latex-beamer', u'latex-xcolor', u'lmodern', u'bibclean',
+            u'pgf', u'preview-latex-style',
+            u'gs-gpl', u'psutils', u'xpdf-utils',
+        )
+
+    def install_mailrelay(self, host, port, user, password, local_email):
+        self.p.install(
+            u'esmtp',
+            u'esmtp-run',
+        )
+        self.p.p.run([u'chown', 'root:mail', '/etc/esmtprc'])
+        self.p.p.run([u'chmod', '0640',      '/etc/esmtprc'])
+        self.p.p.run([u'chown', 'root:mail', '/usr/bin/esmtp'])
+        self.p.p.run([u'chmod', '2755',      '/usr/bin/esmtp'])
+        self.p.p.write(u'/etc/esmtprc', 0640,
+            u'hostname=%s:%i\n'
+            u'username=%s\n'
+            u'password=%s\n'
+            u'starttls=disabled\n'
+            u'mda="/usr/sbin/sendmail -f noreply@`hostname -f` %s"\n'
+            % (host, port, user, password, local_email)
+        )
+
+    def install_postgresql(self):
+        version = u'8.4'
+        self.p.install(
+            u'postgresql-%s'            % (version,),
+            u'postgresql-server-dev-%s' % (version,),
+            u'libpq-dev',
+        )
+        self.p.stop_service(u'postgresql-%s' % (version,))
+        self.p.p.write(u'/etc/postgresql/%s/main/pg_hba.conf' % (version,), 0640,
+            u'local  all  postgres  ident\n'
+            u'local  all  all       ident\n'
+        )
+        postgresql_conf_path = u'/etc/postgresql/%s/main/postgresql.conf' % (version,)
+        self.p.p.write(postgresql_conf_path, 0644,
+            self.p.p.read(postgresql_conf_path) +
+            u'\n'
+            u"listen_addresses = ''\n"
+        )
+        self.p.start_service(u'postgresql-%s' % (version,))
+
+    def add_postgresql_user(self, user, superuser=False, createdb=False, createrole=False):
+        options = u''
+        if superuser:
+            options += u' SUPERUSER'
+        else:
+            options += u' NOSUPERUSER'
+        if createdb:
+            options += u' CREATEDB'
+        else:
+            options += u' NOCREATEDB'
+        if createrole:
+            options += u' CREATEROLE'
+        else:
+            options += u' NOCREATEROLE'
+        self.p.p.run([u'su', u'-', u'postgres', u'-c', u'psql -v ON_ERROR_STOP=1'],
+                     input=u'CREATE USER "%s"%s;' % (user, options))
+
+    def install_webserver(self, default_site):
+        self.p.install(
+            u'awstats', u'libgeo-ipfree-perl',
+            u'apache2-mpm-itk', u'libapache2-mod-fcgid', u'libapache2-svn', 'libapache2-mod-php5',
+            u'php5-cgi', u'php5-cli', u'php5-curl', u'php5-gd', u'php5-imagick', u'php5-imap',
+            u'php5-json', u'php5-ldap', u'php5-mcrypt', u'php5-mhash', u'php5-mysql', u'php5-pgsql',
+            u'php-imlib', u'php-mail-mime', u'php-soap', u'libphp-jpgraph', u'libphp-phplot',
+            u'libcompress-zlib-perl', u'libemail-send-perl', u'libhtml-format-perl',
+            u'libmailtools-perl', u'libnet-dns-perl', u'libnet-ip-perl', u'libnet-smtpauth-perl',
+            u'libwww-perl', u'libxml-perl', u'libxml-parser-perl', u'libxml-libxml-perl',
+            u'python-4suite', u'python-egenix-mxdatetime', u'python-egenix-mxtools', u'python-flup',
+            u'python-ldap', u'python-mysqldb', u'python-pgsql', u'python-psyco', u'python-psycopg2',
+            u'python-pyopenssl', u'python-tz', u'python-unit', u'python-webpy', u'python-xml',
+            u'ruby', u'ruby-dev', u'irb', u'rdoc',
+            u'libfcgi-ruby', u'libyaml-ruby', u'libzlib-ruby', u'libopenssl-ruby', u'liberb-ruby',
+            u'libdbd-mysql-ruby', u'libdbd-pg-ruby', u'libdbd-sqlite3-ruby', u'libtermios-ruby',
+            u'libjson-ruby', u'libreadline-ruby', u'libredcloth-ruby', u'librmagick-ruby',
+        )
+        self.p.stop_service(u'apache2')
+        self.p.p.write(u'/etc/awstats/awstats.conf.local', 0644,
+            u'Lang="auto"\n'
+            u'AllowToUpdateStatsFromBrowser=1\n'
+            u'\n'
+            u'LogFormat=1\n'
+            u'WrapperScript="/"\n'
+            u'DirIcons="icon"\n'
+            u'EnableLockForUpdate=1\n'
+            u'AllowAccessFromWebToAuthenticatedUsersOnly=1\n'
+            u'\n'
+            u'LoadPlugin="geoipfree"\n'
+        )
+        self.p.p.write(u'/etc/apache2/ports.conf', 0644,
+            u'Listen 80\n'
+            u'Listen 443\n'
+        )
+        self.p.p.write(u'/etc/apache2/conf.d/charset', 0644,
+            u'#AddDefaultCharset UTF-8\n'
+        )
+        self.p.p.write(u'/etc/apache2/mods-available/fcgid.conf', 0644,
+            u'<IfModule mod_fcgid.c>\n'
+            u'    AddHandler fcgid-script .fcgi\n'
+            u'    SocketPath /var/lib/apache2/fcgid/sock\n'
+            u'    IPCConnectTimeout 20\n'
+            u'    IPCCommTimeout 30\n'
+            u'</IfModule>\n'
+        )
+        self.p.p.write(u'/etc/php5/conf.d/local_settings.ini', 0644,
+            u'memory_limit = 256M\n'
+        )
+        self.p.set_apache_module_enabled(u'fcgid',   True)
+        self.p.set_apache_module_enabled(u'php5',    True)
+        self.p.set_apache_module_enabled(u'rewrite', True)
+        self.p.set_apache_module_enabled(u'include', True)
+        self.p.set_apache_module_enabled(u'ssl',     True)
+        self.p.start_service(u'apache2')
+
     def install_zope2(self, user, password, zope_user, zope_password):
         self.p.install(
             u'python-4suite',
